package gs.smolban.model.users

import cats.Show
import gs.smolban.model.CreatedAt
import gs.uuid.v0.UUID

/** Represents a user that can interact with Smolban.
  *
  * @param id
  *   Unique identifier of this user.
  * @param createdAt
  *   The instant at which this user was created.
  * @param username
  *   The (unique) [[Username]] of this user.
  * @param userType
  *   The [[UserType]] of this user.
  * @param roles
  *   List of [[Role]] assigned to this user.
  */
case class User(
  id: User.Id,
  createdAt: CreatedAt,
  username: Username,
  userType: UserType,
  roles: List[Role]
)

object User:

  given CanEqual[User, User] = CanEqual.derived

  /** Unique identifier for a [[User]]. This is an opaque type for a UUID.
    */
  opaque type Id = UUID

  object Id:

    /** Instantiate a new [[User.Id]].
      *
      * @param id
      *   The underlying UUID.
      * @return
      *   The new [[User.Id]] instance.
      */
    def apply(id: UUID): Id = id

    given CanEqual[Id, Id] = CanEqual.derived

    given Show[Id] = _.toUUID().withoutDashes()

    extension (id: Id)
      /** Unwrap this User ID.
        *
        * @return
        *   The underlying UUID value.
        */
      def toUUID(): UUID = id

  end Id

end User
